# Auto-generated file, do not edit
from Keywords import Keyword as Kw
Legacy = Kw("Legacy")
Core = Kw("Core")
Development = Kw("Development")
Building = Kw("Building")
Debugger = Kw("Debugger")
IDE = Kw("IDE")
GUIDesigner = Kw("GUIDesigner")
Profiling = Kw("Profiling")
RevisionControl = Kw("RevisionControl")
Translation = Kw("Translation")
Office = Kw("Office")
Calendar = Kw("Calendar")
ContactManagement = Kw("ContactManagement")
Database = Kw("Database")
Dictionary = Kw("Dictionary")
Chart = Kw("Chart")
Email = Kw("Email")
Finance = Kw("Finance")
FlowChart = Kw("FlowChart")
PDA = Kw("PDA")
ProjectManagement = Kw("ProjectManagement")
Presentation = Kw("Presentation")
Spreadsheet = Kw("Spreadsheet")
WordProcessor = Kw("WordProcessor")
Graphics = Kw("Graphics")
x2DGraphics = Kw("2DGraphics")
VectorGraphics = Kw("VectorGraphics")
RasterGraphics = Kw("RasterGraphics")
x3DGraphics = Kw("3DGraphics")
Scanning = Kw("Scanning")
OCR = Kw("OCR")
Photograph = Kw("Photograph")
Viewer = Kw("Viewer")
Settings = Kw("Settings")
DesktopSettings = Kw("DesktopSettings")
HardwareSettings = Kw("HardwareSettings")
PackageManager = Kw("PackageManager")
Network = Kw("Network")
Dialup = Kw("Dialup")
InstantMessaging = Kw("InstantMessaging")
IRCClient = Kw("IRCClient")
FileTransfer = Kw("FileTransfer")
HamRadio = Kw("HamRadio")
News = Kw("News")
P2P = Kw("P2P")
RemoteAccess = Kw("RemoteAccess")
Telephony = Kw("Telephony")
WebBrowser = Kw("WebBrowser")
WebDevelopment = Kw("WebDevelopment")
AudioVideo = Kw("AudioVideo")
Audio = Kw("Audio")
Midi = Kw("Midi")
Mixer = Kw("Mixer")
Sequencer = Kw("Sequencer")
Tuner = Kw("Tuner")
Video = Kw("Video")
TV = Kw("TV")
AudioVideoEditing = Kw("AudioVideoEditing")
Player = Kw("Player")
Recorder = Kw("Recorder")
DiscBurning = Kw("DiscBurning")
Game = Kw("Game")
ActionGame = Kw("ActionGame")
AdventureGame = Kw("AdventureGame")
ArcadeGame = Kw("ArcadeGame")
BoardGame = Kw("BoardGame")
BlocksGame = Kw("BlocksGame")
CardGame = Kw("CardGame")
KidsGame = Kw("KidsGame")
LogicGame = Kw("LogicGame")
RolePlaying = Kw("RolePlaying")
Simulation = Kw("Simulation")
SportsGame = Kw("SportsGame")
StrategyGame = Kw("StrategyGame")
Education = Kw("Education")
Art = Kw("Art")
Construction = Kw("Construction")
Music = Kw("Music")
Languages = Kw("Languages")
Science = Kw("Science")
Astronomy = Kw("Astronomy")
Biology = Kw("Biology")
Chemistry = Kw("Chemistry")
Geology = Kw("Geology")
Math = Kw("Math")
MedicalSoftware = Kw("MedicalSoftware")
Physics = Kw("Physics")
Teaching = Kw("Teaching")
Amusement = Kw("Amusement")
Applet = Kw("Applet")
Archiving = Kw("Archiving")
Electronics = Kw("Electronics")
Emulator = Kw("Emulator")
Engineering = Kw("Engineering")
FileManager = Kw("FileManager")
Shell = Kw("Shell")
Screensaver = Kw("Screensaver")
TerminalEmulator = Kw("TerminalEmulator")
TrayIcon = Kw("TrayIcon")
System = Kw("System")
Filesystem = Kw("Filesystem")
Monitor = Kw("Monitor")
Security = Kw("Security")
Utility = Kw("Utility")
Accessibility = Kw("Accessibility")
Calculator = Kw("Calculator")
Clock = Kw("Clock")
TextEditor = Kw("TextEditor")
Qt = Kw("Qt")
GTK = Kw("GTK")
KDE = Kw("KDE")
GNOME = Kw("GNOME")
Motif = Kw("Motif")
Java = Kw("Java")
ConsoleOnly = Kw("ConsoleOnly")



import sys
import os
import glob
import re

from Keywords import Keyword as Kw, Set as KwS
from Prophet.Categories import *
from Prophet import msg

import Prophet


class App(Prophet.App):

    """A simple application that may have different executables"""
    pref = 30

    def __new__(cls):
        try:
            self = cls.__inst__
            if not self:
                raise Prophet.NotSet

        except AttributeError:
            self = cls.__inst__ = object.__new__(cls)
            try:
                self.__setup__()
            except Prophet.NotSet:
                cls.__inst__ = None
                raise

        return self

    def setKeywords(self):
        super(App, self).setKeywords()
        self.keywords |= KwS(Legacy)

    def setExename(self):
        # Obtain known executable names for the application
        try:
            exes = self.exes
        except AttributeError:
            # If none were explicitly specified, use self class name
            exes = [self.__class__.__name__]
        prefixes = self.getPrefixes()
        paths = self.getPaths()
        valids = []
        for x in exes:
            for pfx, bps in prefixes.binpaths.items():
                try:
                    exe = bps.which(x)
                    if self.valid(pfx, exe):
                        valids.append((pfx, exe))

                except Prophet.NotFound:
                    pass

        try:
            self.prefix, self.exename = self.select(valids)
        except Prophet.NotFound:
            raise Prophet.NotSet

    def valid(self, pfx, exe):
        return True

    def select(self, valids):
        if len(valids):
            return valids[0]
        else:
            raise Prophet.NotFound


class ConsoleApp(Prophet.App):

    """Mixin class for the console application that must be run in terminal"""

    def setKeywords(self):
        super(ConsoleApp, self).setKeywords()
        self.keywords |= KwS(ConsoleOnly)


class X11App(Prophet.App):

    """Mixin class for the X11 GUI application"""


class ZeroG(App):

    """Application installed by the ZeroG LaunchAnywhere system.
        This is usually a commercial Java application"""
    registry = "~/.com.zerog.registry.xml"

    def getPrefixes(self):
        prefixes = super(ZeroG, self).getPrefixes()
        try:
            zerog = open(os.path.expanduser(self.registry), "r")
            pattern = re.compile(
                ".*<product.*name=\"(%s)\".*location=\"(.*)\".*last_modified=\"(.*)\".*>.*" %
                self.magic)
            found = []
            for x in zerog:
                rx = pattern.match(x)
                if rx:
                    found.append((rx.group(3), rx.group(1), rx.group(2)))

            zerog.close()
            found.sort()
            found.reverse()
            prefixes = Prophet.PrefixSet([x[2] for x in found]) + prefixes
        except IOError:
            pass
        return prefixes


class DropIn(App):

    maxDepth = 3
    dropRoots = ["~"]

    class StopDescention(Exception):

        def __init__(self, value):
            self.value = value

    def getPrefixes(self):
        prefixes = super(DropIn, self).getPrefixes()
        try:
            for r in self.dropRoots:
                self.descend(os.path.expanduser(r), 1)

        except DropIn.StopDescention as e:
            prefixes += e.value
        return prefixes

    def descend(self, path, depth):
        self.relevant(path)
        if depth > self.maxDepth:
            return
        try:
            for x in os.listdir(path):
                dir = os.path.join(path, x)
                if not x.startswith(".") and os.path.isdir(dir):
                    self.descend(dir, depth + 1)

        except OSError:
            pass


__legacy__ = ["Development", "Editor", "Emulator",
              "Multimedia", "Graphics", "Network", "Shell"]

entries = []  # List of all legacy entries found


def _register(module, this=True):
    """Import and store the specified module along with all its submodules"""
    try:
        names = module.__legacy__
    except AttributeError:
        names = []
    if this:
        for k, v in module.__dict__.items():
            if not k.startswith("_") and isinstance(
                    v,
                    type) and issubclass(
                    v,
                    Prophet.App):
                entries.append(v)

    for x in names:
        name = module.__name__ + "." + x
        try:
            __import__(name)
        except ImportError:
            raise ImportError("No module named " + name)
        imp = sys.modules[name]
        _register(imp)


def setup():
    _register(sys.modules[__name__], this=False)


def scan():
    result = []
    msg("  legacy...", newline=False)
    for x in entries:
        try:
            result.append(x())
        except Prophet.NotSet:
            pass

    msg(" %d apps found" % len(result))
    return result



import MenuMaker
import Prophet.Legacy

from MenuMaker import indent, writeFullMenu

menuFile = "~/.config/xfce4/desktop/menu.xml"


def _map(x):
    for d, s in (("&amp;", "&"), ("\'", "\"")):
        x = x.replace(s, d)
    return x


class xfss(Prophet.Legacy.App):
    exes = ["xfce-setting-show"]
    name = "Settings"


class xfhelp(Prophet.Legacy.App):
    exes = ["xfhelp4"]
    name = "Help"


class xfabout(Prophet.Legacy.App):
    exes = ["xfce4-about"]
    name = "About"


class Sep(object):

    def emit(self, level):
        return ['%s<separator/>' % indent(level)]


class App(object):

    def emit(self, level):
        if self.app.terminal:
            term = " term=\"true\""
        else:
            term = ""
        return [
            '%s<app name="%s" cmd="%s"%s/>' %
            (indent(level),
             _map(
                self.app.name),
                self.app.execmd,
                term)]


class Menu(object):

    def emit(self, level):
        result = ['%s<menu name="%s">' % (indent(level), _map(self.name))]
        for x in self:
            result += x.emit(level + 1)
        result.append('%s</menu>' % indent(level))
        return result


class Root(object):

    def __init__(self, subs):
        if writeFullMenu:
            subs = subs + [MenuMaker.Sep(), SysMenu()]
        super(Root, self).__init__(subs)

    def emit(self, level):
        if writeFullMenu:
            menu = [
                '<?xml version="1.0" encoding="UTF-8"?>',
                '<!DOCTYPE xfdesktop-menu []>',
                '<xfdesktop-menu>',
                '<title name="Desktop Menu"/>'
            ] + MenuMaker.Sep().emit(level)
            for x in self:
                menu += x.emit(level)
            menu.append('</xfdesktop-menu>')
            return menu
        else:
            menu = []
            for x in self:
                menu += x.emit(level)
            return menu


class SysMenu(MenuMaker.Menu):
    name = "Xfce"

    def __init__(self):
        subs = []
        for cls in (xfss, xfhelp, xfabout):
            try:
                subs.append(MenuMaker.App(cls()))
            except Prophet.NotSet:
                pass

        subs += [
            MenuMaker.Sep(),
            X('<builtin name="Quit" cmd="quit"/>')
        ]
        super(SysMenu, self).__init__(subs)
        self.align = MenuMaker.Entry.StickBottom


class X(MenuMaker.Entry):

    def __init__(self, x):
        super(X, self).__init__()
        self.align = MenuMaker.Entry.StickBottom
        self.x = x

    def emit(self, level):
        return [indent(level) + self.x]



import types
import sys
import os
import os.path
import stat
import string
import fnmatch
import re

from Paths import ValidPathSet as VPS
from Keywords import Set as KwS, Keyword as Kw
from Prophet.Categories import *

verbose = 0
# Omit (whenever possible) the path to the executable if the latter is in
# the PATH
omitExecPath = True
skipMissExec = True  # Skip the entries whose executables are not found


class NotFound(Exception):

    """Raised when an appropriate file is not found"""


class NotSet(Exception):

    """Raised when getter fails to return any meaningful result"""


def msg(s, newline=True):
    if verbose:
        sys.stderr.write(s)
        if newline:
            sys.stderr.write("\n")

    sys.stderr.flush()


def warn(s, newline=True):
    sys.stderr.write(s)
    if newline:
        sys.stderr.write("\n")
    sys.stderr.flush()


def fatal(s, newline=True, code=1):
    sys.stderr.write(s)
    if newline:
        sys.stderr.write("\n")
    sys.stderr.flush()
    sys.exit(code)


def isExe(exe):
    """Check whether the specified name can be considered executable"""
    # Exec's have x-bit set for current uid, aren't directories, and don't
    # have leading dot
    try:
        st = os.stat(exe)
        return (stat.S_IMODE(st[stat.ST_MODE]) & stat.S_IXUSR) and not stat.S_ISDIR(
            st[stat.ST_MODE]) and not os.path.basename(exe).startswith(".")
    except OSError:
        return False


class PathSet(VPS):

    def adopt(self, path):
        path = super(PathSet, self).adopt(path)
        try:
            for exe in os.listdir(path):
                if isExe(os.path.join(path, exe)):
                    return path

        except OSError:
            pass
        raise ValueError

    def which(self, exe):
        if os.path.isabs(exe):
            exe = os.path.basename(exe)
        for p in self:
            x = os.path.join(p, exe)
            if isExe(x):
                return x

        raise NotFound


class PrefixSet(VPS):

    def __init__(self, *args, **kwds):
        self.binpaths = {}
        self.bins = ["bin", "sbin", "", "games"]
        super(PrefixSet, self).__init__(*args, **kwds)

    def adopt(self, path):
        path = super(PrefixSet, self).adopt(path)
        for bin in self.bins:
            dir = os.path.join(path, bin)
            # A valid bindir contains at least one executable
            try:
                for x in os.listdir(dir):
                    if isExe(os.path.join(dir, x)):
                        self.binpaths[path] = PathSet(
                            [os.path.join(path, x) for x in self.bins])
                        return path

            except OSError:
                pass

        raise ValueError


# Set of valid directories containing executables extracted from PATH
paths = PathSet(os.environ["PATH"].split(":"))

# Set of valid prefixes
prefixes = PrefixSet([os.path.dirname(x) for x in paths]) \
    + ["~", "/", "/usr", "/usr/local", "/usr/X11", "/usr/X11R7",
        "/usr/X11R6", "/usr/X11R5", "/opt", "$QTDIR", "$KDEDIR"]

# Match the most widespread shells
# TODO : detect absolute paths
_shStart = re.compile("^\s*(a|b|ba|c|k|pdk|tc|z)?sh\s+")

# Match complex shell command (i.e. with pipelines, and's or's etc.)
_shComplexCmd = re.compile("(;.*){2,}|\||&&|\(|\)|\{|\}")


class App(object):

    """Single application that has specific executable and thus can be put into a menu"""
    Console = Kw("Console")
    XWindow = Kw("XWindow")
    GNOME = Kw("GNOME")
    KDE = Kw("KDE")
    Xfce = Kw("Xfce")
    GNUstep = Kw("GNUstep")

    def getPrefixes(self):
        return prefixes

    def getPaths(self):
        return paths

    def setKeywords(self):
        # Make sure the keywords field always exists
        try:
            self.keywords
        except AttributeError:
            self.keywords = KwS()

    def setName(self):
        try:
            self.name
        except AttributeError:
            # Deduce name from the executable
            self.name = os.path.basename(self.exename).capitalize()

    def setExecmd(self):
        # If an absolute path is given, check it
        # otherwise try to locate the executable via the PATH envar
        path = self.exename
        paths = self.getPaths()
        if os.path.isabs(path):
            if skipMissExec and not isExe(path):
                raise NotSet("%s is not executable" % path)
            if omitExecPath:
                try:
                    # Check whether the specified filename is that one
                    # accessible via the PATH
                    located = paths.which(path)
                    try:
                        if os.path.samefile(path, located):
                            path = os.path.basename(path)

                    except OSError:
                        pass

                except NotFound:
                    pass

        else:
            # Here and forth we're given a relative path
            try:
                located = paths.which(path)
                # There is no reason to check for located<->path identity; we simply
                # make sure the executable can be accessed via PATH
                if not omitExecPath:
                    # Sice we're initially given a relative path, make it full
                    # by consuming the result of locateExec()
                    path = located

            except NotFound:
                if skipMissExec:
                    raise NotSet("executable %s not found in PATH" % path)

        # Execmd is fully formatted command line for the application
        try:
            # exeargs may be missing, it's OK
            self.execmd = path + " " + self.exeargs
        except AttributeError:
            self.execmd = path

    def setTerminal(self):
        try:
            self.terminal
        except AttributeError:
            self.terminal = len(self.keywords & KwS(ConsoleOnly)) > 0

    def setNativenv(self):
        try:
            self.nativenv
        except AttributeError:
            pass
            if self.terminal:
                self.nativenv = App.Console
            else:
                self.nativenv = App.XWindow
            for x in self.keywords:
                if fnmatch.fnmatchcase(x, "*GNOME*"):
                    self.nativenv = App.GNOME
                    break
                elif fnmatch.fnmatchcase(x, "*KDE*"):
                    self.nativenv = App.KDE
                    break
                elif fnmatch.fnmatchcase(x, "*XFCE*"):
                    self.nativenv = App.Xfce
                    break
                # TODO : GNUstep???

        # Attributes to set, mandatory(+), optional(-) :
        # +exename        -- executable, with or w/o path
        # -exeargs        -- arguments that follow the exename
        # +execmd        -- a fully formatted command line for the app
        # +keywords        -- a set of keywords, must not be empty
        # +terminal        -- True if the app requires console support
        # +nativenv        -- the environment required by the app
        # -name            -- short description of the app
        # -comment        -- a more detailed description
        # NOTE : When determining whether the application should be run in terminal,
        # consult the self.terminal and _not_ the self.nativenv

    def testGoodness(self, cmd):
        # Raise NotSet if cmd is not "good enough" and thus isn't worth further consideration
        # Current implementation filters out complex commands and commands
        # that are started with ?sh (a frequent Debian case)
        # There is also a possibility to get None instead of string when the key is missing
        if cmd is None or _shStart.search(cmd) or _shComplexCmd.search(cmd):
            raise NotSet("not good enough : " + str(cmd))

    def __setup__(self):
        self.setExename()
        self.setKeywords()
        try:
            self.setExeargs()
        except AttributeError:
            pass
        self.setExecmd()
        self.setName()
        self.setTerminal()
        self.setNativenv()
        # Skip unwanted entries
        if skip[self.nativenv]:
            raise NotSet("unwanted %s environment" % self.nativenv)
        try:
            self.setComment()
        except AttributeError:
            pass


skip = {App.Console: False, App.XWindow: False,
        App.GNOME: False, App.KDE: False, App.Xfce: False}


def merge(entries):
    """Removes repeating entries from the list using fuzzy matching technique"""
    # NOTE : this is the deliberate violation of the immutability principle
    # __matched attribute is set for the entries that have been successfully matched before
    # Such entries should be skipped
    # It is thought that this approach is faster than keeping them in separate
    # list
    msg("* merging...", newline=False)
    result = []
    esz = len(entries)
    for i in range(0, esz - 1):
        e = entries[i]
        if hasattr(e, "__matched"):
            continue
        matching = [e]  # List containing matching entries
        for j in range(i + 1, esz):
            x = entries[j]
            if hasattr(x, "__matched"):
                continue
            # Matching e and x
            if os.path.basename(e.exename) == os.path.basename(x.exename):
                # The main criteria is the considence of the executables
                try:
                    eargs = e.exeargs
                except AttributeError:
                    eargs = None
                try:
                    xargs = x.exeargs
                except AttributeError:
                    xargs = None
                if not eargs and not xargs:
                    # Both entries don't have arguments, consider them matching
                    matching.append(x)
                elif eargs and xargs:
                    # Both entries do have arguments, match them
                    if Kw(eargs) == Kw(xargs):
                        # Squeeze the spaces and don't consider the case of the
                        # characters
                        matching.append(x)

        if len(matching) < 2:
            # Only one candidate, no choice
            winner = e
        else:
            # Several candidates, must choose the most relevant
            winner = None
            for m in matching:
                if winner:
                    if m.pref > winner.pref:
                        winner = m
                    elif m.pref == winner.pref:
                        # For now the longer name is the better
                        if len(m.name) > len(winner.name):
                            winner = m

                else:
                    winner = m

        result.append(winner)
        # This is done in the last step to mark e which was put in matching
        for m in matching:
            m.__matched = True

    msg(" %d coincidings detected" % (len(entries) - len(result)))
    return result



from Prophet.Legacy import App as _App, ConsoleApp as _ConsoleApp, X11App as _X11App
from Keywords import Keyword as Kw, Set as KwS
from Prophet.Categories import *


class alsamixer(_App, _ConsoleApp):
    name = "ALSA mixer"
    comment = "ALSA sound mixer"
    keywords = KwS(Audio, Mixer)


class alsamixergui(_App, _X11App):
    name = "X ALSA mixer"
    comment = "ALSA sound mixer for X11"
    keywords = KwS(Audio, Mixer)


class alsaplayer(_App, _X11App):
    name = "ALSA player"
    comment = "ALSA audio player for X11"
    keywords = KwS(Audio, Player)

# REASON : requires device to be specified in command line
# class amixer(_App, _ConsoleApp) :
#    name = "Amixer"
#    comment = "ALSA sound mixer"
#    keywords = KwS(Audio, Mixer)


class audacity(_App, _X11App):
    name = "Audacity"
    comment = "Sound editor for X11"
    keywords = KwS(Audio, AudioVideoEditing)


class aumix(_App, _ConsoleApp):
    name = "AUmix"
    comment = "Sound mixer"
    keywords = KwS(Audio, Mixer)


class aviplay(_App, _X11App):
    name = "Aviplay"
    comment = "Vidio player for X11"
    keywords = KwS(Video, Player)


class beep(_App, _X11App):
    name = "Beep"
    comment = "Beep media player, a XMMS fork"
    exes = ["beep-media-player"]
    keywords = KwS(Audio, Player, GTK)


class cheesetracker(_App, _X11App):
    name = "CheeseTracker"
    comment = "Sound tracker similar to the ScreamTracker"
    keywords = KwS(Audio, Player, Sequencer)


class cdp(_App, _ConsoleApp):
    name = "CDp"
    comment = "CD player"
    keywords = KwS(Audio, Player)


class cplay(_App, _ConsoleApp):
    name = "CPlay"
    comment = "Console autio player"
    keywords = KwS(Audio, Player)


class flxine(_App, _X11App):
    name = "FLXINE"
    comment = "XINE media player"
    keywords = KwS(AudioVideo, Audio, Video, TV, Tuner, Player)


class fxtv(_App, _X11App):
    name = "FxTV"
    comment = "*BSD TV tuner"
    keywords = KwS(AudioVideo, Video, Tuner, TV)


class easytag(_App, _X11App):
    name = "EasyTag"
    comment = "Audio file tags manager"
    keywords = KwS(AudioVideo, AudioVideoEditing)


class emusic(_App, _X11App):
    name = "Emusic"
    comment = "Enlightenment media player"
    keywords = KwS(AudioVideo, Audio, Player)


class eroaster(_App, _X11App):
    name = "Eroaster"
    comment = "CD ripper/burner"
    keywords = KwS(Audio, Recorder, DiscBurning)


class _HelixReal(_App, _X11App):
    comment = "Media player"
    keywords = KwS(AudioVideo, Audio, Video, TV, Tuner, Player, GTK)

    def __init__(self):
        try:
            super(_HelixReal, self).__init__()
        except Prophet.NotFound:
            raise


class hxplay(_HelixReal):
    prefix = "HelixPlayer"
    name = "HelixPlayer"


class realplay(_HelixReal):
    prefix = "RealPlayer"
    name = "RealPlayer"


class gcombust(_App, _X11App):
    name = "Gcombust"
    comment = "CD ripper/burner"
    keywords = KwS(Audio, Recorder, DiscBurning)


class gmplayer(_App, _X11App):
    name = "Gmplayer"
    comment = "Mplayer frontend"
    keywords = KwS(AudioVideo, Audio, Video, TV, Tuner, Player, GTK)


class gqmpeg(_App, _X11App):
    name = "GQmpeg"
    comment = "Media player"
    keywords = KwS(AudioVideo, Audio, Video, Player)


class gqcam(_App, _X11App):
    name = "GQcam"
    comment = "Web camera control program"
    keywords = KwS(Video, Network)


class grip(_App, _X11App):
    name = "Grip"
    comment = "CD ripper/burner"
    keywords = KwS(Audio, Recorder, DiscBurning)


class gtv(_App, _X11App):
    name = "Gtv"
    comment = "Vidio player for X11"
    keywords = KwS(Video, Player)


class gxine(_App, _X11App):
    name = "GXINE"
    comment = "XINE media player"
    keywords = KwS(AudioVideo, Audio, Video, TV, Tuner, Player, GTK)


class motv(_App, _X11App):
    name = "MoTV"
    comment = "TV tuner"
    keywords = KwS(AudioVideo, Video, Tuner, TV, Motif)


class mp3shell(_App, _X11App):
    name = "MP3shell"
    comment = "Audio player"
    keywords = KwS(Audio, Player)


class radio(_App, _ConsoleApp):
    name = "Radio"
    comment = "Radio control utility"
    keywords = KwS(Audio, Player, Tuner, Utility)


class rexima(_App, _ConsoleApp):
    name = "Rexima"
    comment = "Sound mixer"
    keywords = KwS(Audio, Mixer)


class x11amp(_App, _X11App):
    name = "X11amp"
    comment = "Audio player, a WinAmp clone"
    keywords = KwS(Audio, Player)


class xanim(_App, _X11App):
    name = "Xanim"
    comment = "X video player"
    keywords = KwS(Video, Player)


class xatitv(_App, _X11App):
    name = "XatiTV"
    comment = "TV tuner control for ATI cards"
    keywords = KwS(AudioVideo, Video, Tuner, TV, Utility)


class xaumix(_App, _X11App):
    name = "Xaumix"
    comment = "Sound mixer, an Aumix frontend"
    keywords = KwS(Audio, Mixer)


class xawtv(_App, _X11App):
    name = "XawTV"
    comment = "TV tuner"
    keywords = KwS(AudioVideo, Video, Tuner, TV)


class xcdroast(_App, _X11App):
    name = "Xcdroast"
    comment = "CD burner"
    keywords = KwS(Audio, DiscBurning)


class xmix(_App, _X11App):
    name = "Xmix"
    comment = "X sound mixer"
    keywords = KwS(Audio, Mixer)


class xmms(_App, _X11App):
    name = "XMMS"
    comment = "X multimedia sound system"
    keywords = KwS(Audio, Player, GTK)


class xmovie(_App, _X11App):
    name = "Xmovie"
    comment = "X video player"
    keywords = KwS(Video, Player)


class xmps(_App, _X11App):
    name = "XMPS"
    comment = "X video player"
    keywords = KwS(Video, Player)


class xpcd(_App, _X11App):
    name = "XPCD"
    comment = "PhotoCD Viewer"
    keywords = KwS(Graphics, Photograph, Viewer)


class zinf(_App, _X11App):
    name = "Zinf"
    comment = "Media player"
    keywords = KwS(AudioVideo, Audio, Video, TV, Tuner, Player, GTK)



import MenuMaker

from MenuMaker import indent

menuFile = "~/.icewm/menu"


def _map(x):
    for d, s in (("\'", "\""),):
        x = x.replace(s, d)
    return x


class App(object):

    def emit(self, level):
        cmd = self.app.execmd
        if self.app.terminal:
            cmd = MenuMaker.terminal.runCmd(cmd)
        return ['%sprog "%s" - %s' % (indent(level), _map(self.app.name), cmd)]


class Menu(object):

    def emit(self, level):
        result = ['%smenu "%s" folder {' % (indent(level), _map(self.name))]
        for x in self:
            result += x.emit(level + 1)
        result.append('%s}' % indent(level))
        return result


class Root(object):

    def emit(self, level):
        menu = []
        for x in self:
            menu += x.emit(level)
        return menu



import os.path
import MenuMaker
import Prophet
import Prophet.Legacy

from MenuMaker import indent, writeFullMenu, omitEmptyMenu

menuFile = "~/GNUstep/Defaults/WMRootMenu"


def _map(x):
    for d, s in (("\'", "\""),):
        x = x.replace(s, d)
    return x


class wmaker(Prophet.Legacy.App):
    pass


class wmsetbg(Prophet.Legacy.App):
    exeargs = "-u -t"


class setstyle(Prophet.Legacy.App):
    pass


class seticons(Prophet.Legacy.App):
    pass


class getstyle(Prophet.Legacy.App):
    exeargs = "-t"


class geticonset(Prophet.Legacy.App):
    exeargs = "-t"


class WPrefs(Prophet.Legacy.App):

    def getPrefixes(self):
        return Prophet.PrefixSet(
            os.path.join(
                wmaker().prefix,
                "lib/GNUstep/Applications/WPrefs.app")) + super(
            WPrefs,
            self).getPrefixes()


class App(object):

    def emit(self, level):
        cmd = self.app.execmd
        if self.app.terminal:
            cmd = MenuMaker.terminal.runCmd(cmd)
        if self._comma:
            return [
                '%s("%s", EXEC, "%s"),' %
                (indent(level),
                 _map(
                    self.app.name),
                    cmd)]
        else:
            return [
                '%s("%s", EXEC, "%s")' %
                (indent(level),
                 _map(
                    self.app.name),
                    cmd)]


class Menu(object):

    def _commas(self):
        if len(self) > 0:
            entries = []
            for x in self:
                if isinstance(x, MenuMaker.Menu):
                    if not (omitEmptyMenu and x.empty):
                        entries.append(x)

                else:
                    entries.append(x)

            if len(entries) > 0:
                for i in range(0, len(entries) - 1):
                    entries[i]._comma = True
                entries[len(entries) - 1]._comma = False
            return entries
        else:
            return []

    def emit(self, level):
        if len(self) > 0:
            menu = ['%s("%s",' % (indent(level), _map(self.name))]
        else:
            menu = ['%s("%s"' % (indent(level), _map(self.name))]
        for x in self._commas():
            menu += x.emit(level + 1)
        if self._comma:
            menu.append('%s),' % indent(level))
        else:
            menu.append('%s)' % indent(level))
        return menu


class Root(object):
    _comma = False
    name = "WindowMaker"

    def __init__(self, subs):
        if writeFullMenu:
            subs += [SysMenu()]
        super(Root, self).__init__(subs)

    def emit(self, level):
        if writeFullMenu:
            return super(Root, self).emit(level)
        else:
            menu = []
            for x in self._commas():
                menu += x.emit(level)
            return menu


class SysMenu(MenuMaker.Menu):
    name = "WindowMaker"

    def __init__(self):
        subs = [
            X("Workspaces", "WORKSPACE_MENU"),
            AppearanceMenu(),
            X("Run...", "SHEXEC", "%a(Run,Type command to run:)"),
            X("Save session", "SAVE_SESSION"),
            X("Clear session", "CLEAR_SESSION")
        ]
        try:
            subs.append(X("Preferences", "EXEC", WPrefs().execmd))
        except Prophet.NotSet:
            pass
        subs += [
            X("Restart", "RESTART"),
            X("Exit", "EXIT")
        ]
        super(SysMenu, self).__init__(subs)
        self.align = MenuMaker.Entry.StickBottom


class AppearanceMenu(MenuMaker.Menu):
    name = "Appearance"

    def __init__(self):
        subs = []
        try:
            sys = os.path.join(wmaker().prefix, "share/WindowMaker/")
        except Prophet.NotSet:
            sys = "/"  # Doesn't actually matter
        usr = "$HOME/GNUstep/Library/WindowMaker/"
        try:
            execmd = setstyle().execmd
            for id in ("Themes", "Styles"):
                subs += self.openMenu(id, execmd, [sys + id, usr + id])

        except Prophet.NotSet:
            pass
        try:
            subs += self.openMenu("Icons", seticons().execmd,
                                  [sys + "IconSets", usr + "IconSets"])
        except Prophet.NotSet:
            pass
        try:
            subs += self.openMenu("Backgrounds", wmsetbg().execmd,
                                  [sys + "Backgrounds", usr + "Backgrounds"])
        except Prophet.NotSet:
            pass
        try:
            subs.append(X("Save Theme", "SHEXEC", getstyle(
            ).execmd + " " + usr + '''Themes/\\"%a(Theme name)\\"'''))
        except Prophet.NotSet:
            pass
        try:
            subs.append(X("Save IconSet", "SHEXEC", geticonset(
            ).execmd + " " + usr + '''IconSets/\\"%a(IconSet name)\\"'''))
        except Prophet.NotSet:
            pass
        super(AppearanceMenu, self).__init__(subs)
        self.align = MenuMaker.Entry.StickBottom

    def openMenu(self, name, execmd, dirs):
        s = ""
        for x in dirs:
            dir = os.path.expandvars(x)
            if os.path.isdir(dir):
                s += x + " "

        if len(s) > 0:
            return [X(name, "OPEN_MENU", "-noext " + s + "WITH " + execmd)]
        else:
            return []


class X(MenuMaker.Entry):

    def __init__(self, name, cmd, arg=None):
        super(X, self).__init__()
        self.name = name
        self.cmd = cmd
        self.arg = arg
        self.align = MenuMaker.Entry.StickBottom

    def emit(self, level):
        if self.arg:
            s = '%s("%s", %s, "%s")' % (
                indent(level), _map(self.name), self.cmd, self.arg)
        else:
            s = '%s("%s", %s)' % (indent(level), _map(self.name), self.cmd)
        if self._comma:
            return [s + ","]
        else:
            return [s]



import os.path
import MenuMaker
import Prophet
import Prophet.Legacy

from MenuMaker import indent, writeFullMenu

menuFile = "~/.pekwm/menu"


def _map(x):
    for d, s in (("\'", "\""),):
        x = x.replace(s, d)
    return x


class pekwm(Prophet.Legacy.App):
    pass


class Sep(object):

    def emit(self, level):
        return ['%sSeparator {}' % indent(level)]


class App(object):

    def emit(self, level):
        cmd = self.app.execmd
        if self.app.terminal:
            cmd = MenuMaker.terminal.runCmd(cmd)
        return [
            '%sEntry = "%s" { Actions = "Exec %s" }' %
            (indent(level),
             _map(
                self.app.name),
                cmd)]


class Menu(object):
    tag = "Submenu"

    def emit(self, level):
        result = ['%s%s = "%s" {' % (indent(level), self.tag, _map(self.name))]
        for x in self:
            result += x.emit(level + 1)
        result.append('%s}' % indent(level))
        return result


class Root(object):
    name = "/"
    tag = "Rootmenu"

    def __init__(self, subs):
        if writeFullMenu:
            subs += [MenuMaker.Sep(), SysMenu()]
        super(Root, self).__init__(subs)

    def emit(self, level):
        if writeFullMenu:
            return super(Root, self).emit(level) + [windowMenu]
        else:
            result = []
            for x in self:
                result += x.emit(level)
            return result


class SysMenu(MenuMaker.Menu):
    name = "PekWM"

    def __init__(self):
        subs = [
            ThemesMenu(),
            X("Reload", "Reload"),
            X("Restart", "Restart"),
            MenuMaker.Sep(),
            X("Exit", "Exit")
        ]
        super(SysMenu, self).__init__(subs)
        self.align = MenuMaker.Entry.StickBottom


class ThemesMenu(MenuMaker.Menu):
    name = "Themes"

    def __init__(self):
        subs = []
        try:
            prefix = pekwm().prefix
            themeset = os.path.join(
                prefix, "share/pekwm/scripts/pekwm_themeset.pl")
            if Prophet.isExe(themeset):
                for dir in (
                        os.path.join(
                            prefix,
                            "share/pekwm/themes"),
                        "~/.pekwm/themes"):
                    if os.path.isdir(os.path.expanduser(dir)):
                        subs.append(X(None, "Dynamic %s %s" % (themeset, dir)))

        except Prophet.NotSet:
            pass
        super(ThemesMenu, self).__init__(subs)
        self.align = MenuMaker.Entry.StickBottom


class X(MenuMaker.Entry):

    def __init__(self, entry, actions):
        super(X, self).__init__()
        self.entry = entry
        self.actions = actions
        self.align = MenuMaker.Entry.StickBottom

    def emit(self, level):
        if self.entry:
            return [
                '%sEntry = "%s" { Actions = "%s" }' %
                (indent(level),
                 _map(
                    self.entry),
                    self.actions)]
        else:
            return [
                '%sEntry { Actions = "%s" }' %
                (indent(level), self.actions)]


windowMenu = '''
WindowMenu = "Window Menu" {
    Entry = "(Un)Stick" { Actions = "Toggle Sticky" }
    Entry = "(Un)Shade" { Actions = "Toggle Shaded" }
    Submenu = "Maximize" {
        Entry = "Full" { Actions = "Toggle Maximized True True" }
        Entry = "Horizontal" { Actions = "Toggle Maximized True False" }
        Entry = "Vertical" { Actions = "Toggle Maximized False True" }
    }
    Submenu = "Fill" {
        Entry = "Full" { Actions = "MaxFill True True" }
        Entry = "Horizontal" { Actions = "MaxFill True False" }
        Entry = "Vertical" { Actions = "MaxFill False True" }
    }
    Submenu = "Stacking" {
        Entry = "Raise " { Actions = "Raise" }
        Entry = "Lower" { Actions = "Lower" }
        Entry = "Always On Top " { Actions = "Toggle AlwaysOnTop" }
        Entry = "Always Below" { Actions = "Toggle AlwaysBelow" }
    }
    Submenu = "Decor" {
        Entry = "Decor" { Actions = "Toggle DecorBorder; Toggle DecorTitlebar" }
        Entry = "Border" { Actions = "Toggle DecorBorder" }
        Entry = "Titlebar" { Actions = "Toggle DecorTitlebar" }
    }
    Submenu = "Skip" {
        Entry = "Menus" { Actions = "Toggle Skip Menus" }
        Entry = "Focus Toggle" { Actions = "Toggle Skip FocusToggle" }
        Entry = "Snap" { Actions = "Toggle Skip Snap" }
    }
    Entry = "Iconify " { Actions = "Set Iconified" }
    Entry = "Manual Action" { Actions = "ShowCmdDialog" }
    Separator {}
    Entry = "Close" { Actions = "Close" }
    Entry = "Kill " { Actions = "Kill " }
}
'''
